=== modified file 'client/errname.cc'
--- client/errname.cc	2009-11-27 21:14:56 +0000
+++ client/errname.cc	2009-12-11 21:21:33 +0000
@@ -715,6 +715,7 @@
     { "ER_WARNING_NON_DEFAULT_VALUE_FOR_VIRTUAL_COLUMN",1679 },
     { "ER_UNSUPPORTED_ACTION_ON_VIRTUAL_COLUMN",1680 },
     { "ER_CONST_EXPR_IN_VCOL",1681 },
+    { "ER_NO_PRIMARY_KEY_ON_REPLICATED_TABLE", 1692},
     { 0, 0 }
   };
 

=== modified file 'drizzled/cursor.cc'
--- drizzled/cursor.cc	2009-12-04 20:52:44 +0000
+++ drizzled/cursor.cc	2009-12-11 21:21:33 +0000
@@ -1359,6 +1359,8 @@
   if (table->s->tmp_table || ! replication_services.isActive())
     return false;
 
+  bool result= false;
+
   switch (session->lex->sql_command)
   {
   case SQLCOM_REPLACE:
@@ -1394,7 +1396,7 @@
     else
     {
       if (before_record == NULL)
-        replication_services.insertRecord(session, table);
+        result= replication_services.insertRecord(session, table);
       else
         replication_services.updateRecord(session, table, before_record, after_record);
     }
@@ -1408,7 +1410,7 @@
      * an update.
      */
     if (before_record == NULL)
-      replication_services.insertRecord(session, table);
+      result= replication_services.insertRecord(session, table);
     else
       replication_services.updateRecord(session, table, before_record, after_record);
     break;
@@ -1424,7 +1426,7 @@
     break;
   }
 
-  return false;
+  return result;
 }
 
 int Cursor::ha_external_lock(Session *session, int lock_type)

=== modified file 'drizzled/replication_services.cc'
--- drizzled/replication_services.cc	2009-12-09 17:09:29 +0000
+++ drizzled/replication_services.cc	2009-12-11 21:21:33 +0000
@@ -58,6 +58,7 @@
 #include "drizzled/message/table.pb.h"
 #include "drizzled/gettext.h"
 #include "drizzled/session.h"
+#include "drizzled/error.h"
 
 #include <vector>
 
@@ -328,7 +329,7 @@
 }
 
 message::Statement &ReplicationServices::getInsertStatement(Session *in_session,
-                                                            Table *in_table) const
+                                                                 Table *in_table) const
 {
   message::Statement *statement= in_session->getStatementMessage();
 
@@ -384,10 +385,23 @@
   }
 }
 
-void ReplicationServices::insertRecord(Session *in_session, Table *in_table)
+bool ReplicationServices::insertRecord(Session *in_session, Table *in_table)
 {
   if (! is_active)
-    return;
+    return false;
+  /**
+   * We do this check here because we don't want to even create a 
+   * statement if there isn't a primary key on the table...
+   *
+   * @todo
+   *
+   * Multi-column primary keys are handled how exactly?
+   */
+  if (in_table->s->primary_key == MAX_KEY)
+  {
+    my_error(ER_NO_PRIMARY_KEY_ON_REPLICATED_TABLE, MYF(0));
+    return true;
+  }
 
   message::Statement &statement= getInsertStatement(in_session, in_table);
 
@@ -411,6 +425,7 @@
     record->add_insert_value(string_value->c_ptr(), string_value->length());
     string_value->free();
   }
+  return false;
 }
 
 message::Statement &ReplicationServices::getUpdateStatement(Session *in_session,

=== modified file 'drizzled/replication_services.h'
--- drizzled/replication_services.h	2009-12-04 16:59:27 +0000
+++ drizzled/replication_services.h	2009-12-11 21:21:33 +0000
@@ -294,8 +294,11 @@
    *
    * @param Pointer to the Session which has inserted a record
    * @param Pointer to the Table containing insert information
+   *
+   * Grr, returning "true" here on error because of the cursor
+   * reversed bool return crap...fix that.
    */
-  void insertRecord(Session *in_session, Table *in_table);
+  bool insertRecord(Session *in_session, Table *in_table);
   /**
    * Creates a new UpdateRecord GPB message and pushes it to
    * replicators.

=== added file 'plugin/transaction_log/tests/r/no_primary_key.result'
--- plugin/transaction_log/tests/r/no_primary_key.result	1970-01-01 00:00:00 +0000
+++ plugin/transaction_log/tests/r/no_primary_key.result	2009-12-11 21:21:33 +0000
@@ -0,0 +1,7 @@
+DROP TABLE IF EXISTS t1;
+CREATE TABLE t1 (
+id INT NOT NULL
+, padding VARCHAR(200) NOT NULL
+);
+INSERT INTO t1 VALUES (1, "I love testing.");
+ERROR HY000: Tables which are replicated require a primary key.

=== added file 'plugin/transaction_log/tests/t/no_primary_key-master.opt'
--- plugin/transaction_log/tests/t/no_primary_key-master.opt	1970-01-01 00:00:00 +0000
+++ plugin/transaction_log/tests/t/no_primary_key-master.opt	2009-12-11 21:21:33 +0000
@@ -0,0 +1,1 @@
+--default-replicator-enable --transaction-log-enable --scheduler=multi_thread

=== added file 'plugin/transaction_log/tests/t/no_primary_key.test'
--- plugin/transaction_log/tests/t/no_primary_key.test	1970-01-01 00:00:00 +0000
+++ plugin/transaction_log/tests/t/no_primary_key.test	2009-12-11 21:21:33 +0000
@@ -0,0 +1,23 @@
+# 
+# Simple test of the transaction log and the errors
+# which should be produced when a table is inserted
+# into when the transaction log is enabled but the
+# table does not have primary keys.
+# 
+# We create a table without a primary key and insert 
+# a record into it.  An error should be produced.
+#
+
+--disable_warnings
+DROP TABLE IF EXISTS t1;
+--enable_warnings
+
+CREATE TABLE t1 (
+  id INT NOT NULL
+, padding VARCHAR(200) NOT NULL
+);
+
+# The below should produce an error because the CREATE TABLE
+# above does not have a primary key.
+--error ER_NO_PRIMARY_KEY_ON_REPLICATED_TABLE
+INSERT INTO t1 VALUES (1, "I love testing.");

